*==============================================================================
* Program:						VFP WinBar
* Purpose:						For quick access to opened windows
* VFP Version					9.0 (SP1)
* e-mail:						canab.ua@gmail.com
* Copyright:					(c) 2006 Yaroslav Kihtan
*==============================================================================

*-- winbar settings
#define WBAR_POSITION 3					&& doc position, 0 - top, 3 - bottom
#define WBAR_THEMES .f.
#define WBAR_FONTSIZE 8
#define WBAR_FONTBOLD .f.				
#define WBAR_FONTNAME 'Tahoma'
#define WBAR_FORECOLOR rgb(0,0,0)

*-- api declarations
#define GW_CHILD			5
#define GW_HWNDNEXT			2

#define WM_SYSCOMMAND		0x0112
#define WM_ACTIVATE			0x0006

#define SW_NORMAL			1
#define SW_MAXIMIZE			3
#define SW_MINIMIZE			6

#define SC_MINIMIZE			0xF020
#define SC_MAXIMIZE			0xF030
#define SC_RESTORE			0xF120
#define SC_CLOSE			0xF060

#define WA_ACTIVE			1

#define SWP_NOSIZE			0x0001
#define SWP_NOMOVE			0x0002
#define SWP_NOZORDER		0x0004
#define SWP_NOREDRAW		0x0008
#define SWP_NOACTIVATE		0x0010
#define SWP_FRAMECHANGED	0x0020  /* The frame changed: send WM_NCCALCSIZE */
#define SWP_SHOWWINDOW		0x0040
#define SWP_HIDEWINDOW		0x0080
#define SWP_NOCOPYBITS		0x0100
#define SWP_NOOWNERZORDER	0x0200  /* Don't do owner Z ordering */
#define SWP_NOSENDCHANGING	0x0400  /* Don't send WM_WINDOWPOSCHANGING */
#define SWP_DRAWFRAME		SWP_FRAMECHANGED
#define SWP_NOREPOSITION	SWP_NOOWNERZORDER

#define HWND_TOP			0
#define HWND_BOTTOM			1
#define HWND_TOPMOST		-1
#define HWND_NOTOPMOST		-2


declare integer ShowWindow in win32api integer hWnd, integer nCmdShow
declare integer GetWindow in win32api integer hWnd, integer uCmd
declare integer GetWindowText in win32api integer hwnd, string @lpString, integer nMaxCount
declare integer SendMessage in win32api integer hwnd, integer Msg, integer wParam, integer lparam
declare integer IsIconic in win32api integer hwnd
declare integer IsWindowVisible in win32api integer hwnd
declare integer IsZoomed in win32api integer hwnd
declare integer SetWindowPos in win32api integer hWnd, integer hWndInsertAfter,;
	integer X, integer Y, integer cx, integer cy, integer uFlags

declare integer GetFocus in win32api
declare integer SetFocus in win32api integer hwnd

*-- create winbar
public go_winbar
if vartype(go_winbar) == 'O' and not isnull(go_winbar)
	go_winbar.release
else
	go_winbar = createobject("cs_winbar")
	go_winbar.show
endif

*==========================================
define class cs_winbar as toolbar
*==========================================
	borderstyle = 0
	themes = WBAR_THEMES
	movable = .f.
	cPath = ''
	nButtonIndex = 0
	dimension aWin[1,3]
	dimension aPicture[1]
	cSkipNames = ''

	add object timer1 as timer with interval = 200
	procedure timer1.timer
		this.parent.updateBar
	endproc

	add object cmd_screen as checkbox with;
		style = 1,;
		width = 20,;
		caption = '',;
		specialeffect = 2,;
		height = 23,;
		tooltiptext = 'Hide All'

	add object separator1 as separator with;
		style = 1

*----------------------------
	procedure cmd_screen.click
*----------------------------
	local i
	if this.value = 1
		with this.parent
			for i=.nButtonIndex to .ControlCount
				=SendMessage(.controls[i].nHandle, WM_SYSCOMMAND, SC_MINIMIZE, 0)
			endfor
		endwith
	else
		with this.parent
			for i=.nButtonIndex to .ControlCount
				if isIconic(.controls[i].nHandle)=1
					=SendMessage(.controls[i].nHandle, WM_SYSCOMMAND, SC_RESTORE, 0)
				endif
			endfor
		endwith
	endif
	this.Parent.updateBar

*----------------------------
	procedure init
*----------------------------
	this.cSkipNames = ;
		'%Form Controls%Report Controls%Layout%Color Palette%Standard%Print Preview%'+;
		'%View Designer%Query Designer%Report Designer%Form Designer%Database Designer%'
	this.nButtonIndex = this.ControlCount + 1
	this.cPath = justpath(getWordNum(sys(16),3))
	this.cmd_screen.picture = this.cPath + '\' + 'screen.bmp'
	this.readPictures
	this.dock(WBAR_POSITION)

*----------------------------
	procedure readPictures
*----------------------------
	local array aFile[1]
	local ln_fcount, i
	
	ln_fcount = adir(aFile, this.cPath+'\*.bmp', 'AHRS')
	for i=1 to ln_fcount
		declare this.aPicture[i]
		this.aPicture[i] = upper(juststem(aFile[i,1]))
	endfor
	asort(this.aPicture, 1, alen(this.aPicture), 1)

*----------------------------
	procedure show
*----------------------------
	dodefault()
	this.updateBar
	
*----------------------------
	procedure updateBar
*----------------------------
	local ln_handle, ln_winCount, i, lc_text, lc_buf, ln_length
	lc_buf = space(200)

	*--
	* scan windows
	*--
	i = 0
	ln_handle = GetWindow(_screen.HWnd, GW_CHILD)
	do while ln_handle > 0

		*-- skip invisible
		if IsWindowVisible(ln_handle) = 0
			ln_handle = GetWindow(ln_handle, GW_HWNDNEXT)
			loop
		endif
		
		*-- skip system toolbars
		ln_length = GetWindowText(ln_handle, @lc_buf, 200)
		lc_text = left(lc_buf, ln_length)
		if '%'+lc_text+'%' $ this.cSkipNames
			ln_handle = GetWindow(ln_handle, GW_HWNDNEXT)
			loop
		endif
		
		i = i + 1
		if i>alen(this.aWin, 1)
			declare this.aWin[i,3]
		endif
		this.aWin[i,1] = ln_handle
		this.aWin[i,2] = lc_text
		this.aWin[i,3] = .t.

		ln_handle = GetWindow(ln_handle, GW_HWNDNEXT)
	enddo
	ln_winCount = i

	*--
	* scan buttons
	*--
	local lo_button, ln_ind, ln_active
	*ln_active = GetFocus()
	ln_active = this.aWin[1]
	
	i = this.nButtonIndex
	do while i<= this.ControlCount
		lo_button = this.Controls(i)
		
		*-- search window in array
		ln_ind = ascan(this.aWin, lo_button.nHandle, 1, ln_winCount, 1, 8)
		if ln_ind = 0
			this.RemoveObject(lo_button.name)
			loop
		endif
		
		this.aWin[ln_ind,3] = .f.
		ln_handle = this.aWin[ln_ind, 1]

		*-- check window state
		if IsIconic(ln_handle) = 1
			=SetWindowPos(ln_handle, HWND_BOTTOM, -100,-100,0,0, SWP_NOSIZE + SWP_NOACTIVATE)
		endif
		if lo_button.value # (lo_button.nHandle = ln_active)
			lo_button.value = (lo_button.nHandle = ln_active)
		endif
		
		*-- check title changed
		if not lo_button.cTitle == this.aWin[ln_ind, 2]
			lo_button.cTitle = this.aWin[ln_ind, 2]
			lo_button.setCaption
			lo_button.Width = lo_button.Width && resize toolbar
		endif
		
		i = i + 1
	enddo

	*--
	* add new buttons
	*--
	for i=1 to ln_winCount
		if not this.aWin[i,3]
			loop
		endif
		this.addObject('H'+transform(this.aWin[i,1]), 'cs_winbarButton')
		lo_button = this.Controls(this.ControlCount)
		with lo_button
			.nHandle = this.aWin[i,1]
			.cTitle = this.aWin[i,2]
			.value = (lo_button.nHandle = ln_active)
			.setCaption
			.setPicture
			.visible = .t.
		endwith
	endfor

*----------------------------
	procedure dblclick
*----------------------------
	nodefault

enddefine

*==========================================
define class cs_winbarButton as checkbox
*==========================================
	fontName = WBAR_FONTNAME
	fontSize = WBAR_FONTSIZE
	fontBold = WBAR_FONTBOLD
	foreColor = WBAR_FORECOLOR
	picturePosition = 1
	pictureSpacing = 1
	pictureMargin = 1
	style = 1
	autosize = .t.
	cTitle = ''
	nHandle = 0

*-----------------------------
	procedure setCaption
*-----------------------------
	local ln_pos1, ln_pos2

	this.Caption = this.cTitle
	this.tooltiptext = this.cTitle
	
	if 'Properties - ' $ this.cTitle
		this.caption = 'Properties'
		return
	endif

	if 'Document View' $ this.cTitle
		this.Caption = 'Document View'
		return
	endif
		
	if 'Class Browser' $ this.cTitle
		ln_pos1 = at('(', this.cTitle)
		if ln_pos1 > 0
			this.Caption = substr(this.cTitle, 1, ln_pos1-1)
			return
		endif
	endif

	ln_pos1 = at('(', this.cTitle)
	ln_pos2 = at(')', this.cTitle)
	if ln_pos1>0 and ln_pos2>0 and ln_pos2>ln_pos1
		this.caption = substr(this.cTitle, ln_pos1+1, ln_pos2-ln_pos1-1)
		return
	endif

	ln_pos1 = at(' - ', this.cTitle)
	if ln_pos1>0
		this.caption = substr(this.cTitle, ln_pos1+3)
		return
	endif

*-----------------------------
	procedure setPicture
*-----------------------------
	local i
	for i=1 to alen(this.parent.aPicture)
		if this.parent.aPicture[i] $ upper(this.cTitle)
			this.Picture = this.parent.cPath + '\' + this.parent.aPicture[i] + '.bmp'
			return
		endif
	endfor
	
*-----------------------------
	procedure click
*-----------------------------
	if not this.Value
		=SendMessage(this.nHandle, WM_SYSCOMMAND, SC_MINIMIZE, 0)
	else
		if IsIconic(this.nHandle) = 1
			=SendMessage(this.nHandle, WM_SYSCOMMAND, SC_RESTORE, 0)
		endif
		=SendMessage(this.nHandle, WM_ACTIVATE, WA_ACTIVE, 0)
	endif
	this.parent.updateBar

*-----------------------------
	procedure rightclick
*-----------------------------
	=SendMessage(this.nHandle, WM_SYSCOMMAND, SC_CLOSE, 0)
	this.parent.updateBar
	
enddefine
